# -*- encoding: utf-8 -*-
import os
import json
import sys
import yaml

from suite_py.lib.singleton import Singleton
from suite_py.lib.logger import Logger

logger = Logger()


class Config(metaclass=Singleton):
    _config = {}
    _config_path_file = os.path.join(os.environ["HOME"], ".suite_py/config.yml")
    _base_cache_path = os.path.join(os.environ["HOME"], ".suite_py/cache")

    def __init__(self):
        if not os.path.exists(self._base_cache_path):
            os.makedirs(self._base_cache_path)

    def load(self):
        with open(self._config_path_file) as configfile:
            self._config = yaml.safe_load(configfile)

        self._config["user"]["projects_home"] = os.path.join(
            os.environ["HOME"], self._config["user"]["projects_home"]
        )

        self._config["user"].setdefault("review_channel", "#review")

        self._config["user"].setdefault("deploy_channel", "#deploy")

        self._config["user"].setdefault("default_slug", "PRIMA-XXX")

        self._config["user"].setdefault("captainhook_timeout", 30)  # This is in seconds

        self._config["user"].setdefault(
            "captainhook_url", "http://captainhook-internal.prima.it"
        )

        self.load_local_config()

        return self._config

    def load_local_config(self):
        local_conf_path = os.path.join(os.curdir, ".suite_py.yml")
        try:
            with open(local_conf_path) as f:
                local_conf = yaml.safe_load(f)

        except FileNotFoundError:
            return

        for key, value in local_conf["user"].items():
            self._config["user"][key] = value

    def put_cache(self, key, data):
        with open(os.path.join(self._base_cache_path, key), "w") as cache_file:
            json.dump(data, cache_file)

    def get_cache(self, key):
        try:
            with open(os.path.join(self._base_cache_path, key)) as cache_file:
                return json.load(cache_file)
        except Exception:
            logger.error(
                f"Non ho trovato nessuna versione in cache per la chiave {key}. Attiva la VPN."
            )
            sys.exit(-1)



# -*- coding: utf-8 -*-
import sys

from suite_py.lib.handler.youtrack_handler import YoutrackHandler
from suite_py.lib.handler import git_handler as git
from suite_py.lib.logger import Logger
from suite_py.lib.qainit import qainit_shutdown


youtrack = YoutrackHandler()
logger = Logger()


def entrypoint(project):
    branch_name = git.current_branch_name(project)

    logger.info("Spengo il qa se esite...")
    qainit_shutdown(branch_name)

    sys.exit()



# -*- coding: utf-8 -*-

from suite_py.lib.handler.captainhook_handler import CaptainHook
from suite_py.lib.handler.github_handler import GithubHandler
from suite_py.lib.handler.youtrack_handler import YoutrackHandler
from suite_py.lib.handler.slack_handler import SlackHandler
from suite_py.lib.handler import drone_handler as drone
from suite_py.lib.handler import prompt_utils
from suite_py.lib.tokens import Tokens
from suite_py.lib.config import Config
from suite_py.lib.logger import Logger

logger = Logger()
config = Config()
tokens = Tokens()
youtrack = YoutrackHandler()
github = GithubHandler()
slack = SlackHandler()
captainhook = CaptainHook()

CHECKMARK = "✔"
CROSSMARK = "✘"
INVALID_OR_MISSING_TOKEN = []


def entrypoint(timeout):
    if timeout:
        captainhook.set_timeout(timeout)

    # Services
    github_status = check_github()
    slack_status = check_slack()
    drone_status = check_drone()
    youtrack_status = check_youtrack()
    captainhook_status = check_captainhook()

    forge_message("Github", github_status)
    forge_message("Slack", slack_status)
    forge_message("Drone", drone_status)
    forge_message("Youtrack", youtrack_status)
    forge_message("Captainhook", captainhook_status)

    # Tokens
    if len(INVALID_OR_MISSING_TOKEN) > 0 and prompt_utils.ask_confirm(
        "Vuoi reinserire i token mancanti?", default=True
    ):
        reiterate_token()


def check_github():
    if tokens.github:
        try:
            if github.get_user().login:
                return "ok"
            INVALID_OR_MISSING_TOKEN.append("github")
            return "invalid_token"
        except Exception:
            INVALID_OR_MISSING_TOKEN.append("github")
            return "invalid_token"
    else:
        INVALID_OR_MISSING_TOKEN.append("github")
        return "missing_token"


def check_slack():
    if tokens.slack:
        try:
            if slack.get_user()["ok"]:
                return "ok"
            INVALID_OR_MISSING_TOKEN.append("slack")
            return "invalid_token"
        except Exception:
            INVALID_OR_MISSING_TOKEN.append("slack")
            return "invalid_token"
    else:
        INVALID_OR_MISSING_TOKEN.append("slack")
        return "missing_token"


def check_drone():
    if tokens.drone:
        try:
            drone_user = drone.get_user()
            if "message" in drone_user and drone_user["message"] == "Unauthorized":
                INVALID_OR_MISSING_TOKEN.append("drone")
                return "invalid_token"
            return "ok"
        except Exception:
            INVALID_OR_MISSING_TOKEN.append("drone")
            return "invalid_token"
    else:
        INVALID_OR_MISSING_TOKEN.append("drone")
        return "missing_token"


def check_youtrack():
    if tokens.youtrack:
        try:
            youtrack.get_projects()
            return "ok"
        except Exception:
            INVALID_OR_MISSING_TOKEN.append("youtrack")
            return "invalid_token"
    else:
        INVALID_OR_MISSING_TOKEN.append("youtrack")
        return "missing_token"


def check_captainhook():
    try:
        if captainhook.check().status_code != 200:
            return "unreachable"
        return "ok"
    except Exception:
        return "unreachable"


def forge_message(service, result):
    cases = {
        "ok": f"{service:>12}:{CHECKMARK:>12} ok",
        "invalid_token": f"{service:>12}:{CROSSMARK:>12} token non valido",
        "missing_token": f"{service:>12}:{CROSSMARK:>12} token mancante",
        "unreachable": f"{service:>12}:{CROSSMARK:>12} non raggiungibile",
    }
    print(cases.get(result, f"{service}: stato sconosciuto"))


def reiterate_token():
    for service in INVALID_OR_MISSING_TOKEN:
        new_token = prompt_utils.ask_questions_input(
            f"Inserisci nuovo token per {service}: "
        )
        tokens.edit(service, new_token)

    logger.info("Salvo i nuovi token...")
    tokens.save()
    logger.info(f"{CHECKMARK} Fatto!")



# -*- coding: utf-8 -*-
import os
from os import listdir
from os.path import isfile, join
import stat
import sys
import subprocess
from datetime import datetime
import yaml
from jinja2 import Environment, DebugUndefined, FileSystemLoader
from suite_py.lib.handler.github_handler import GithubHandler
from suite_py.lib.handler import git_handler as git
from suite_py.lib.handler import prompt_utils
from suite_py.lib.logger import Logger
from suite_py.lib.config import Config
from suite_py.lib.handler import drone_handler as drone

github = GithubHandler()
logger = Logger()
config = Config()

now = datetime.now().strftime("%Y%m%d%H%M")

# define paths
projects_root = config.load()["user"]["projects_home"]
artemide_path = os.path.join(projects_root, "artemide")
drone_values_path = f"{artemide_path}/templates/drone/values/"
deploy_scripts_values_path = f"{artemide_path}/templates/deploy_scripts/values/"


def entrypoint():
    # move to artemide root directory
    os.chdir(artemide_path)

    env = Environment(
        loader=FileSystemLoader(artemide_path),
        undefined=DebugUndefined,
        trim_blocks=True,
        lstrip_blocks=True,
    )

    template_type = prompt_utils.ask_choices(
        "Quali file vuoi generare?", ["drone", "deploy_scripts"]
    )
    message = f"Autogenerated update {template_type} from suite-py {now}"

    autobranch = prompt_utils.ask_confirm(
        "Vuoi creare branch e pr in automatico?", default=False
    )

    if autobranch:
        autopush = True
    else:
        autopush = prompt_utils.ask_confirm(
            "Vuoi pushare in automatico?", default=False
        )

    choice = prompt_utils.ask_questions_input(
        "Su quale progetto vuoi lavorare? Inserisci `all` per selezionarli tutti: ",
        "all",
    )

    if choice == "all":
        projects = get_all_templated_projects(template_type)
    else:
        projects = [choice]

    for project in projects:
        logger.info(f"{project}: Controllo che non ci siano file non committati")
        if git.is_dirty(project):
            logger.error(
                f"{project}: Ci sono modifiche non committate, skippo il progetto"
            )
            continue

        project_root_path = os.path.join(projects_root, project)
        branch_name = configure_branch(project, template_type, autobranch, autopush)

        logger.info(f"{project}: Generazione in corso")
        # launch generate
        if template_type == "drone":
            generate_drone(project_root_path, f"{drone_values_path}{project}.yml", env)
        elif template_type == "deploy_scripts":
            generate_deploy_scripts(
                project_root_path, f"{deploy_scripts_values_path}{project}.yml", env
            )
        logger.info(f"{project}: Generazione completata")

        if autobranch or autopush:
            # controllo che sia cambiato effettivamente qualcosa
            if git.is_dirty(project):
                git_operations(project, autobranch, autopush, branch_name, message)
            else:
                logger.warning(
                    f"{project}: nessuna operazione su git da fare. Nessun file modificato"
                )


def get_all_templated_projects(template_type):
    if template_type == "drone":
        values_path = drone_values_path
    elif template_type == "deploy_scripts":
        values_path = deploy_scripts_values_path

    return [
        f.replace(".yml", "")
        for f in listdir(values_path)
        if isfile(join(values_path, f))
    ]


def configure_branch(project, template_type, autobranch, autopush):
    logger.info(f"{project}: Sincronizzo il repo con il remote")
    if autobranch:
        git.sync(project)  # fa anche checkout su master
        return f"t/autogenerated_update_{template_type}_{now}"

    current_branch = git.current_branch_name(project)
    if autopush and git.remote_branch_exists(project, current_branch):
        git.fetch(project)
        git.pull(project)

    return current_branch


def generate_drone(project_path, values_file, env):
    values = yaml.safe_load(open(values_file))
    template = env.get_template(f"templates/drone/{values['template']}.j2")
    rendered = template.render(values)

    write_on_repo(project_path, rendered, ".drone.yml")
    drone.fmt(values["project_name"], project_path)
    drone.validate(values["project_name"], project_path)
    drone.sign(values["project_name"], project_path)


def generate_deploy_scripts(project_path, values_file, env):
    values = yaml.safe_load(open(values_file))
    template = env.get_template(f"templates/deploy_scripts/{values['template']}.j2")

    rendered = template.render(values)
    file_name = "deploy"
    write_on_repo(os.path.join(project_path, "deploy"), rendered, file_name)
    validate_deploy_script(f"{project_path}/{file_name}")

    if values["cloudformation"]:
        template = env.get_template("templates/deploy_scripts/wait_for_stack.j2")
        rendered = template.render(values)
        file_name = "wait_for_stack"
        write_on_repo(os.path.join(project_path, "deploy"), rendered, file_name)
        validate_deploy_script(f"{project_path}/{file_name}")


def write_on_repo(path, rendered, file_name):
    if not os.path.exists(path):
        os.mkdir(path, mode=0o755)

    fd = open(os.path.join(path, file_name), "w")
    fd.write(rendered)
    fd.close()


def fmt(project_path):
    subprocess.run(
        ["drone", "fmt", "--save", ".drone.yml"], cwd=project_path, check=True
    )


def validate_drone(project_path):
    try:
        subprocess.run(
            ["drone", "lint", "--trusted", ".drone.yml"], cwd=project_path, check=True
        )
    except Exception:
        logger.error(f"Template in path {project_path} is not valid")
        sys.exit(-1)


def make_executable(filepath):
    st = os.stat(filepath)
    os.chmod(filepath, st.st_mode | stat.S_IEXEC)


def validate_deploy_script(filepath):
    make_executable(filepath)
    subprocess.run(
        ["shellcheck", filepath], check=True,
    )


def sign(project_path, project):
    subprocess.run(
        ["drone", "sign", f"primait/{project}", "--save"], cwd=project_path, check=True
    )


def git_operations(project, autobranch, autopush, branch_name, message):
    if autobranch:
        logger.info(f"{project}: creo il branch {branch_name}")
        git.checkout(project, branch_name, new=True)
        git.add(project)
        git.commit(project, message)
        git.push(project, branch_name)
        pr = github.create_pr(project, branch_name, message)
        logger.info(f"Pull request numero {pr.number} creata! {pr.html_url}")
    elif autopush:
        if git.current_branch_name(project) != "master":
            git.add(project)
            git.commit(project, message)
            git.push(project, branch_name)
        else:
            logger.warning(
                f"{project} si trova su master, non eseguo il push automatico"
            )



# -*- coding: utf-8 -*-
import re

from pptree import Node, print_tree
from suite_py.lib.handler import aws_handler as aws
from suite_py.lib.logger import Logger

logger = Logger()


def entrypoint(project_name, env):

    clusters_names = aws.get_ecs_clusters(env)
    n_services = Node("services")

    projects = {
        "prima": ["web", "consumer"],
        "ab_normal": ["abnormal"],
    }
    project_names = projects.get(project_name, [project_name])

    for cluster_name in clusters_names:

        services = []
        all_services = aws.get_ecs_services(cluster_name)

        for service in all_services:
            if service["status"] == "ACTIVE":
                for prj in project_names:
                    if prj in service["serviceName"]:
                        services.append(service["serviceName"])

        for service in services:
            container_instances = []
            container_instances = aws.get_container_instances_arn_from_service(
                cluster_name, service
            )
            if container_instances:
                ips = aws.get_ips_from_container_instances(
                    cluster_name, container_instances
                )

                m = re.search(f"ecs-task-.*-{env}-ECSService(.*)-.*", service)
                if m:
                    n_service = Node(m.group(1), n_services)
                    for ip in ips:
                        Node(ip, n_service)

    print_tree(n_services, horizontal=True)
    logger.info("Done!")



# -*- encoding: utf-8 -*-
import slack

from suite_py.lib.tokens import Tokens
from suite_py.lib.singleton import Singleton
from suite_py.lib.logger import Logger
from suite_py.lib.config import Config

tokens = Tokens()
config = Config()
logger = Logger()


class SlackHandler(metaclass=Singleton):
    _client = None

    def __init__(self):
        self._client = slack.WebClient(token=tokens.slack)

    def get_user(self):
        return self._client.users_profile_get()

    def post(self, channel_config, text):
        channel = config.load()["user"].get(channel_config)
        if channel:
            try:
                return self._client.chat_postMessage(
                    channel=channel, text=text, as_user=True
                )
            except Exception:
                logger.warning("Non sono riuscito a inviare il messaggo su Slack")
        else:
            logger.info("Le notifiche su Slack sono disabilitate")

        return None

    def reply_thread(self, channel_config, thread_id, text):
        channel = config.load()["user"].get(channel_config)
        if channel:
            try:
                return self._client.chat_postMessage(
                    channel=channel, thread_ts=thread_id, text=text, as_user=True
                )
            except Exception:
                logger.warning("Non sono riuscito a inviare il messaggo su Slack")
        else:
            logger.info("Le notifiche su Slack sono disabilitate")

        return None
